#!/usr/bin/python

import sys, string
import matplotlib.pyplot as plt
import numpy as np
import math

n = 101
on = 1.0/(n-1)

sg    = np.zeros(n)
rho   = np.zeros(n)
lamda = np.zeros(n)
mu    = np.zeros(n)
lmr   = np.zeros(n)
k     = np.zeros(n)
vs    = np.zeros(n)
vp    = np.zeros(n)
vpvsr = np.zeros(n)
nu    = np.zeros(n)
ai    = np.zeros(n)
si    = np.zeros(n)

porosity= 0.33

rhorock  = 2500
rhowater = 1000
rhogas   = 100
rho0 = (1.0-porosity)*rhorock + porosity*rhowater
rho1 = (1.0-porosity)*rhorock + porosity*rhogas

vsref  = 1200
muref  = rho0 * vsref * vsref
 
# Gassman
# https://en.wikipedia.org/wiki/Gassmann%27s_equation
# Too complicated ...


# Quick and Dirty:
# Lamda(SG) = Lamda(SG=0) + (Lamda(SG=1)-Lamda(SG=0)*exp(-SG*FactorLamda)
# SG = Gas Saturation
lamda0 = muref * 2
lamda1 = muref / 2

fl     = 40.0

i = 0
for i in range(n):
	sg[i]    = on*i
	rho[i]   = rho0 + (rho1-rho0)*sg[i]
	lamda[i] = lamda1 + (lamda0-lamda1)*math.exp(-sg[i]*fl)
	mu[i]    = muref
	k[i]     = lamda[i] + mu[i]*2/3
	vs[i]    = math.sqrt(mu[i]/rho[i])
	vp[i]    = math.sqrt((lamda[i]+2*mu[i])/rho[i])
	vpvsr[i] = vp[i]/vs[i]
	lmr[i]   = lamda[i]/mu[i]
	nu[i]    = lamda[i]/(2*lamda[i]+mu[i])
	ai[i]    = rho[i]*vp[i]
	si[i]    = rho[i]*vs[i]

print ("Gas Saturation : % 8.1f ---> % 8.1f Gas/(Gas+Water)" %(sg[0],sg[n-1]))
print ("Density        : % 8.1f ---> % 8.1f Kg/Meter^3" %(rho[0],rho[n-1]))
print ("Vp             : % 8.1f ---> % 8.1f Meter/Second" %(vp[0],vp[n-1]))
print ("Vs             : % 8.1f ---> % 8.1f Meter/Second" %(vs[0],vs[n-1]))
print ("Vp/Vs    Ratio : % 8.4f ---> % 8.4f" %(vp[0]/vs[0],vp[n-1]/vs[n-1]))
print ("Poisson  Ratio : % 8.4f ---> % 8.4f" %(nu[0],nu[n-1]))
print ("Lamda/Mu Ratio : % 8.4f ---> % 8.4f" %(lmr[0],lmr[n-1]))


"""
print "sg    = ",sg
print "rho   = ",rho
print "lamda = ",lamda
print "mu    = ",mu
print "k     = ",k
print "vs    = ",vs
print "vp    = ",vp
print "vpvsr = ",vpvsr
print "nu    = ",nu
"""

plt.plot(sg,rho)
plt.xlabel('Gas Saturation')
plt.ylabel('Kg/Meter^3')
plt.title('Density')
plt.grid(True)
plt.show()

plt.plot(sg,lamda,c='red')
plt.plot(sg,mu,c='green')
plt.plot(sg,k,c='black')
plt.xlabel('Gas Saturation')
plt.ylabel('Pa')
plt.title('Red=Lamda Green=Mu Black=K')
plt.grid(True)
plt.show()

plt.plot(sg,vs,c='black')
plt.plot(sg,vp,c='red')
plt.xlabel('Gas Saturation')
plt.ylabel('meters/second')
plt.title('Vp & Vs')
plt.grid(True)
plt.show()

plt.plot(sg,si,c='black')
plt.plot(sg,ai,c='red')
plt.xlabel('Gas Saturation')
plt.ylabel('Kg/second/Meter^2')
plt.title('Impedances Red=Acoustic, Black=Shear')
plt.grid(True)
plt.show()

plt.plot(sg,vpvsr,c='red')
plt.plot(sg,nu,c='green')
plt.plot(sg,lmr,c='black')
plt.xlabel('Gas Saturation')
plt.ylabel('Ratio')
plt.title('Red = Vp/Vs Ratio, Green = Poisson Ratio, Black=LMR')
plt.grid(True)
plt.show()
